#define F_CPU 14745600ul

#include <stdlib.h>
#include <math.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "lcd.h"

#define adc8307_start -85.85				// start point [dBm]
#define adc_resolution 4095.0				//
#define adc_ref	2500000.0					// LT 1009 2.5V U Ref
#define adc8307_slope 24800.0				// scaling factor uV/dB
#define average_count	8192				// number of reading averaged

#define CLK PC0								//CLK LT1286
#define CS 	PC1								//CS LT1286
#define DATA PC2							//DATA LT1286

int MAX187_read(void);
float ad8307_p(unsigned int adc_value);

int main(void)
{
	unsigned n = 0;
	char value[9];

	unsigned detector_typ;						// ADC 8307 -> 3
	unsigned error;

	const char title1[]= "mw-Meter DL2FZN ";
	const char title2[]= "01/2012         ";
	char message[16] = "";

	unsigned int adc_pwr;						// ADC raw power
	double pwr;									// power [mW]
	double pwr_dbm;								// power [dBm]
	unsigned long average_sum;					//

	// Port A LCD only output
	DDRA = 0b11111111;
	
	//Port B empty
	DDRB = 0b00000000;

	// Port C ADC PC 0 + PC1 output
	DDRC  = 0b00000011;

	// internal pullup resitors off
	PORTC = 0b00000000;

	//Port D select detector
	DDRD = 0b00000000;

	//Port D PD0 + PD1 pull up on
	PORTD = 0b00000011;

    /* initialize display, cursor off */
    lcd_init(LCD_DISP_ON);

	/* clear display and home cursor */
	lcd_clrscr();

    /* put string to display (line 1) with linefeed */
    lcd_puts(title1);

    /* put string to display (line 2) with linefeed */
	lcd_gotoxy(0,1);
    lcd_puts(title2);

	_delay_ms(5000);

	// Watchdog 500ms
	wdt_enable(WDTO_500MS);

    for (;;)
    {
		average_sum = 0ul;
		
		for (n = 0; n < average_count;n ++)
		{
			average_sum = average_sum + MAX187_read();
		}
		adc_pwr= average_sum / average_count;

		detector_typ = (PIND & 0b00000011);

		if (detector_typ == 3)
		{
			pwr_dbm = ad8307_p(adc_pwr);
			error = 0;
		}
		else
		{
			error = 2;							//detector not defined
		}

		pwr = pow(10, (pwr_dbm/10));

		if (error == 0)
		{
			dtostrf(pwr_dbm,6,1,value);
			/* move cursor to position 0 on line 1 */
			lcd_gotoxy(0,0);
			sprintf(message,"P = %5s dBm  ", value);
			lcd_puts(message);

			if (pwr < 0.000001)
			{
				pwr = pwr * 1000000000.0;
				dtostrf(pwr,4,1,value);
				lcd_gotoxy(0,1);
				sprintf(message,"P = %6s pW   ", value);
				lcd_puts(message);
			}

			else if (pwr < 0.001)
			{
				pwr = pwr * 1000000.0;
				dtostrf(pwr,4,1,value);
				lcd_gotoxy(0,1);
				sprintf(message,"P = %6s nW   ", value);
				lcd_puts(message);
			}
			else if (pwr < 1.0)
			{
				pwr = pwr * 1000.0;
				dtostrf(pwr,4,1,value);
				lcd_gotoxy(0,1);
				sprintf(message,"P = %6s uW   ", value);
				lcd_puts(message);
			}
			else
			{
				dtostrf(pwr,4,1,value);
				lcd_gotoxy(0,1);
				sprintf(message,"P = %6s mW   ", value);
				lcd_puts(message);
 			}
		}
		else if (error == 1)
		{
			lcd_clrscr();
		    lcd_puts("Not valid");
		}
		else if (error == 2)
		{
			lcd_clrscr();
		    lcd_puts("Wrong detector");
			lcd_gotoxy(0,1);
			sprintf(message,"Error = %2u", error);
			lcd_puts(message);
		}


		wdt_reset();			//Reset Watchdog
	}
}

int MAX187_read(void)
{
	int adc_value = 0;							//12bit value from MAX187
	int bit_count = 0;							//Counter for bit

	PORTC |= (1<<CS);							//CS inactive
	PORTC &=~(1<<CLK);							//Reset CLK
	PORTC &=~(1<<CS);							//CS activ
	_delay_us(9);

	for (bit_count = 0; bit_count < 13; bit_count ++)
	{
		PORTC |= (1<<CLK);						//Set CLK

		adc_value = adc_value << 1;				//shift left

		PORTC &=~(1<<CLK);						//Reset CLK

		adc_value = adc_value + (PINC & 0b00000100);

	}
	PORTC |= (1<<CS);							//CS inactive

	adc_value = adc_value >> 2;					//shift right

	adc_value = (adc_value & 0x0fff);			//only 12bits

	return (adc_value);	
}

float ad8307_p(unsigned int adc_value)
{
	float p_dbm;

	p_dbm = (float) adc8307_start + (float) adc_value / adc_resolution * adc_ref / adc8307_slope;
	
	return p_dbm;
}


